<?php
/**
 * This class is responsible to manage all jupiters plugin.
 *
 * @package JupiterX_Core\Control_Panel
 */

class JupiterX_Control_Panel_Install_Plugins {

	protected $tgmpa;
	protected $api_url       = 'http://artbees.net/api/v2/';
	protected $theme_name    = 'JupiterX';
	private static $instance = null;

	/**
	 * Class constructor.
	 *
	 * @since 1.9.0
	 */
	public function __construct() {
		if ( ! class_exists( 'TGM_Plugin_Activation' ) ) {
			return;
		}

		$menu_items_access = get_site_option( 'menu_items' );
		if ( is_multisite() && ! isset( $menu_items_access['plugins'] ) && ! current_user_can( 'manage_network_plugins' ) ) {
			return;
		}

		$this->tgmpa = isset( $GLOBALS['tgmpa'] ) ? $GLOBALS['tgmpa'] : TGM_Plugin_Activation::get_instance();

		add_filter( 'jupiterx_control_panel_pane_install_plugins', [ $this, 'view' ] );

		add_action( 'init', [ $this, 'cache' ] );

		add_action( 'wp_ajax_abb_get_plugins', [ $this, 'get_plugins_for_frontend' ] );
		add_action( 'wp_ajax_abb_deactivate_plugin', [ $this, 'deactivate' ] );
		add_action( 'wp_ajax_abb_update_plugin_checker', [ $this, 'plugin_conflict_checker' ] );
	}

	/**
	 * Load view from Jupiter X Core plugin.
	 *
	 * @since 1.9.0
	 */
	public function view() {
		return jupiterx_core()->plugin_dir() . 'includes/control-panel/views/install-plugins.php';
	}

	/**
	 * Get class instance.
	 *
	 * @since 1.9.0
	 */
	public static function get_instance() {
		if ( ! self::$instance ) {
			self::$instance 	= new self;
		}

		return self::$instance;
	}

	/**
	 * Send a json list of plugins and their data and activation limit status for front-end usage.
	 *
	 * @since 1.9.0
	 */
	public function get_plugins_for_frontend() {
		$plugins = $this->get_all_plugins();

		$plugins = $this->update_plugins_status( $plugins );

		ksort( $plugins );

		$limit = $this->plugins_threshold();

		return wp_send_json( [ 'plugins' => $plugins, 'limit' => $limit ] );
	}

	/**
	 * Get full list of plugins.
	 * 12h cache.
	 *
	 * @since 1.9.0
	 */
	public function get_all_plugins() {
		$plugins = get_transient( 'jupiterx_list_of_plugins' );

		if ( ! $plugins ) {
			$plugins = $this->get_free_plugins() + $this->get_pro_plugins();
			set_transient( 'jupiterx_list_of_plugins' , $plugins, 12 * HOUR_IN_SECONDS );
		}

		return $plugins;
	}

	/**
	 * Get list of pro plugins from Artbees API.
	 *
	 * @since 1.9.0
	 */
	public function get_pro_plugins() {
		return $this->get_plugins_from_api( [ 'slug', 'basename', 'version', 'name', 'desc', 'more_link', 'img_url', 'required', 'pro', 'is_callable' ] );
	}

	/**
	 * Filter tgmpa plugins to extract list of free plugins.
	 *
	 * @since 1.9.0
	 */
	public function get_free_plugins_raw() {
		return array_filter(
			$this->tgmpa->plugins,
			function( $plugin_info, $plugin ) {
				return 'repo' === $plugin_info['source'];
			},
			ARRAY_FILTER_USE_BOTH
		);
	}

	/**
	 * Get list of free plugins.
	 * List gets update to add needed info like icon and version from WordPress repo.
	 *
	 * @since 1.9.0
	 */
	public function get_free_plugins() {

		$plugins = $this->get_free_plugins_raw();

		foreach ( $plugins as $plugin => $plugin_info ) {
			$wp_plugin_info = $this->get_plugin_info_from_wp( $plugin, [ 'icons' => true, 'short_description' => true ] );

			$plugins[ $plugin ]['version'] = $wp_plugin_info->version;
			$plugins[ $plugin ]['desc']    = $wp_plugin_info->short_description;
			$plugins[ $plugin ]['img_url'] = isset( $wp_plugin_info->icons['1x'] ) ? $wp_plugin_info->icons['1x'] : $wp_plugin_info->icons['default'];
		}

		return $plugins;
	}

	/**
	 * Get a list of slugs of free plugins.
	 *
	 * @since 1.9.0
	 */
	public function get_free_plugins_slug() {
		$free_plugins = $this->get_free_plugins();

		return array_column( $plugins, 'slug' );
	}

	/**
	 * Get detail of given plugin from WordPress repo.
	 * Used to get version, icon and plugin description.
	 *
	 * @since 1.9.0
	 *
	 * @param string $plugin_slug Plugin slug from plugin header.
	 * @param array $fields       Needed extra information.
	 */
	public function get_plugin_info_from_wp( $plugin_slug = '', $fields = [] ) {

		if ( empty( $plugin_slug )  ) {
			return;
		}

		if ( ! function_exists( 'plugins_api' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin-install.php';
		}

		$default_fields = [
			'short_description' => false,
			'description'       => false,
			'sections'          => false,
			'tested'            => false,
			'requires'          => false,
			'rating'            => false,
			'downloaded'        => false,
			'downloadlink'      => false,
			'last_updated'      => false,
			'added'             => false,
			'tags'              => false,
			'compatibility'     => false,
			'homepage'          => false,
			'versions'          => false,
			'donate_link'       => false,
			'reviews'           => false,
			'banners'           => false,
			'icons'             => false,
			'active_installs'   => false,
			'group'             => false,
			'contributors'      => false,
		];

		$fields = wp_parse_args( $fields, $default_fields );

		// @todo Set cache.
		$plugin_info = plugins_api(
			'plugin_information',
			[
				'slug'   => $plugin_slug,
				'fields' => $fields,
			]
		);

		if ( is_wp_error( $plugin_info ) ) {
			return [];
		}

		return $plugin_info;
	}

	/**
	 * Get custom list of plugins from Artbees API.
	 *
	 * @param array $fields Needed information.
	 */
	public function get_plugins_from_api( $fields = [] ) {

		$plugin_info = [];

		$free_plugins = array_column( $this->get_free_plugins(), 'slug' );

		$response = wp_remote_get(
			$this->api_url . 'tools/plugin-custom-list',
			[
			'sslverify' => false,
			'headers'   =>
				[
					'domain'       => $_SERVER['SERVER_NAME'],
					'theme-name'   => $this->theme_name,
					'list-of-attr' => wp_json_encode( $fields ),
					'from'         => 0,
					'count'        => 0,
				],
			]
		);

		$plugins = json_decode( wp_remote_retrieve_body( $response ), true );
		$plugins = $plugins['data'];

		$plugins = array_filter(
			$plugins,
			function( $v, $k ) use ( $free_plugins ) {
				return ! in_array( $v['slug'], $free_plugins, true );
			},
			ARRAY_FILTER_USE_BOTH
		);

		$corrected_plugins_list = [];

		foreach ( $plugins as $key => $plugin ) {
			$corrected_plugins_list[ $plugin['slug'] ] = $plugin;
		}

		return $corrected_plugins_list;
	}

	/**
	 * Check number of activated plugins in two different groups.
	 *
	 * @since 1.9.0
	 *
	 * @return bool $threshold Wether we are meeting threshold or not.
	 */
	public function plugins_threshold() {

		$plugins   = get_option('active_plugins');
		$threshold = [];

		if ( count( $plugins ) >= 20 ) {
			$threshold[] = 'num';
		}

		$sliders = [
			'LayerSlider/layerslider.php',
			'masterslider/masterslider.php',
			'revslider/revslider.php',
		];

		if ( count( array_intersect( $plugins, $sliders ) ) >= 1 ) {
			$threshold[] = 'sliders';
		}

		$jet_plugins = [
			'jet-blog/jet-blog.php',
			'jet-elements/jet-elements.php',
			'jet-engine/jet-engine.php',
			'jet-menu/jet-menu.php',
			'jet-popup/jet-popup.php',
			'jet-smart-filters/jet-smart-filters.php',
			'jet-tabs/jet-tabs.php',
			'jet-tricks/jet-tricks.php',
			'jet-woo-builder/jet-woo-builder.php',
		];

		if ( count( array_intersect( $plugins, $jet_plugins ) ) >= 4 ) {
			$threshold[] = 'jet-plugins';
		}

		return implode( $threshold, ',' );
	}

	/**
	 * Delete plugin information transient to load updated one.
	 * Usage: add force-check=1 to the URL of control panel.
	 *
	 * @since 1.9.0
	 */
	public function cache() {
		if ( jupiterx_get( 'force-check', false ) ) {
			delete_transient( 'jupiterx_list_of_plugins' );
		}
	}

	/**
	 * Update plugin information to add activation, installation and update status to plugin data.
	 * URL used to add activation/installation URL using TGMPA.
	 *
	 * @since 1.9.0
	 *
	 * @param array $plugins List of plugins.
	 */
	public function update_plugins_status( $plugins = [] ) {

		foreach ( $plugins as $slug => $plugin ) {

			if ( ! isset( $plugins[ $slug ]['basename'] ) || empty( $plugins[ $slug ]['basename'] ) ) {
				$plugins[ $slug ]['basename'] = $this->find_plugin_path( $slug );
			}

			$plugins[ $slug ]['update_needed']    = false;
			$plugins[ $slug ]['installed']        = false;
			$plugins[ $slug ]['active']           = false;
			$plugins[ $slug ]['network_active']   = false;
			$plugins[ $slug ]['install_disabled'] = false;

			if ( is_plugin_active_for_network( $plugins[ $slug ]['basename'] )  ) {
				if ( ! current_user_can( 'manage_network_plugins' ) ) {
					unset( $plugins[ $slug ] );
					continue;
				}

				$plugins[ $slug ]['network_active'] = true;
			}

			if ( $this->tgmpa->does_plugin_have_update( $slug ) ) {
				$plugins[ $slug ]['update_needed'] = true;
				$plugins[ $slug ]['update_url'] = $this->get_tgmpa_action_url( $slug, 'update' );
			}

			if ( $this->tgmpa->is_plugin_active( $slug ) ) {
				$plugins[ $slug ]['active']    = true;
				$plugins[ $slug ]['installed'] = true;
			} elseif ( $this->tgmpa->is_plugin_installed( $slug ) ) {
				$plugins[ $slug ]['installed'] = true;
			}

			if ( ! jupiterx_is_pro() && 'true' === $plugins[ $slug ]['pro'] && ! $plugins[ $slug ]['installed'] ) {
				$plugins[ $slug ]['pro'] = true;
			} else {
				unset( $plugins[ $slug ]['pro'] );
			}

			if ( ! $plugins[ $slug ]['installed'] && ( is_multisite() && ! current_user_can( 'manage_network_plugins' ) ) ) {
				$plugins[ $slug ]['install_disabled'] = true;
			}

			if ( ! $plugins[ $slug ]['installed'] && ! $plugins[ $slug ]['install_disabled'] ) {
				$plugins[ $slug ]['url'] = $this->get_tgmpa_action_url( $slug, 'install' );
			} elseif ( $plugins[ $slug ]['installed'] && ! $plugins[ $slug ]['active'] ) {
				$plugins[ $slug ]['url'] = $this->get_tgmpa_action_url( $slug, 'activate' );
			}

			if ( $plugins[ $slug ]['installed'] ) {
				$plugin_data                 = get_plugin_data( trailingslashit( WP_PLUGIN_DIR ) . $this->find_plugin_path( $slug ) );
				$plugins[ $slug ]['version'] = $plugin_data['Version'];
			}

		}

		return $plugins;
	}

	/**
	 * Get plugin basename by plugin slug.
	 * Works only for installed plugins.
	 *
	 * @since 1.9.0
	 *
	 * @param string $plugin_slug
	 */
	public function find_plugin_path( $plugin_slug = '' ) {

		$plugins = get_plugins();
		foreach ( $plugins as $plugin_address => $plugin_data ) {

			// Extract slug from address
			if ( strlen( $plugin_address ) == basename( $plugin_address ) ) {
				$slug = strtolower( str_replace( '.php', '', $plugin_address ) );
			} else {
				$slug = strtolower( str_replace( '/' . basename( $plugin_address ), '', $plugin_address ) );
			}
			// Check if slug exists
			if ( strtolower( $plugin_slug ) == $slug ) {
				return $plugin_address;
			}
		}

		return false;
	}

	/**
	 * Get installation/activation URL of a plugin using TGMPA.
	 *
	 * @since 1.9.0
	 *
	 * @param string $slug   Plugin slug.
	 * @param string $action install/activate
	 */
	public function get_tgmpa_action_url( $slug = '', $action = '' ) {
		if ( ! in_array( $action, [ 'install', 'activate', 'update' ], true ) ) {
			wp_send_json_error( [ 'message' => esc_html__( 'Action is not valid.', 'jupiterx-core' ) ] );
		}

		$nonce_url = wp_nonce_url(
			add_query_arg(
				[
					'plugin'           => urlencode( $slug ),
					'tgmpa-' . $action => $action . '-plugin',
				],
				admin_url( 'themes.php?page=tgmpa-install-plugins' )
			),
			'tgmpa-' . $action,
			'tgmpa-nonce'
		);

		return $nonce_url;
	}

	/**
	 * Deactivate plugin using native WordPress functionalities.
	 *
	 * @since 1.9.0
	 */
	public function deactivate() {
		if ( ! isset( $_POST['slug'] ) ) {
			wp_send_json_error( [ 'message' => esc_html__( 'Can\'t deactivate plugin', 'jupiterx-core' ) ] );
		}

		$plugin = $this->find_plugin_path( sanitize_text_field( $_POST['slug'] ) );

		if ( ! current_user_can( 'activate_plugin', $plugin ) ) {
			wp_send_json_error( esc_html__( 'Sorry, you are not allowed to deactivate this plugin.', 'jupiterx-core' ) );
		}

		deactivate_plugins( $plugin );

		wp_send_json_success( esc_html__( 'Deactivated Successfully.', 'jupiterx-core' ) );
	}

	/**
	 * Check for possible conflicts with Themes & Plugins for a specific plugin.
	 *
	 * @since 1.9.0
	 *
	 * @return void
	 */
	public function plugin_conflict_checker() {
		if ( ! function_exists( 'get_plugins' ) ) {
			require_once ABSPATH . 'wp-admin/includes/plugin.php';
		}

		$plugin_data = wp_unslash( $_POST['plugin'] );

		if ( empty( $plugin_data ) ) {
			wp_json_send_error( __( 'Plugin data is missing', 'jupiterx' ) );
		}

		if ( 'wp-repo' === $plugin_data['version'] ) {
			$wp_updated_plugins = get_site_transient('update_plugins');

			if (
				empty( $wp_updated_plugins ) &&
				empty( $wp_updated_plugins->response[ $plugin_data['basename'] ] )
			) {
				wp_send_json_success();
			}

			$plugin_data['version'] = $wp_updated_plugins
				->response[ $plugin_data['basename'] ]
				->new_version;
		}

		$conflicts = jupiterx_get_plugin_conflicts( $plugin_data, get_plugins() );

		if ( count( $conflicts['plugins'] ) > 0 || count( $conflicts['themes'] ) > 0 ) {
			wp_send_json_error( $conflicts );
		}

		wp_send_json_success();
	}
}

JupiterX_Control_Panel_Install_Plugins::get_instance();
